# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_schema[admin] 1'] = {
    'data': {
        '__schema': {
            'mutationType': {
                'fields': [
                    {
                        'name': 'authenticateWithGitHub'
                    },
                    {
                        'name': 'createLog'
                    },
                    {
                        'name': 'createProduct'
                    },
                    {
                        'name': 'deleteProduct'
                    },
                    {
                        'name': 'updateProduct'
                    },
                    {
                        'name': 'convertProductType'
                    },
                    {
                        'name': 'createProductFilePath'
                    },
                    {
                        'name': 'deleteProductFilePath'
                    },
                    {
                        'name': 'updateProductFilePath'
                    },
                    {
                        'name': 'createProductRelation'
                    },
                    {
                        'name': 'deleteProductRelation'
                    },
                    {
                        'name': 'createProductRelationTypes'
                    },
                    {
                        'name': 'deleteProductRelationTypes'
                    },
                    {
                        'name': 'updateProductRelationType'
                    },
                    {
                        'name': 'createProductType'
                    },
                    {
                        'name': 'deleteProductType'
                    },
                    {
                        'name': 'updateProductType'
                    },
                    {
                        'name': 'createField'
                    },
                    {
                        'name': 'deleteField'
                    },
                    {
                        'name': 'updateField'
                    },
                    {
                        'name': 'addGitHubOrg'
                    },
                    {
                        'name': 'deleteGitHubOrg'
                    },
                    {
                        'name': 'addGitHubAdminAppToken'
                    },
                    {
                        'name': 'deleteGitHubAdminAppToken'
                    },
                    {
                        'name': 'updateGitHubOrgMemberLists'
                    },
                    {
                        'name': 'deleteLog'
                    },
                    {
                        'name': 'saveWebConfig'
                    }
                ]
            },
            'queryType': {
                'fields': [
                    {
                        'name': 'webConfig'
                    },
                    {
                        'name': 'isSignedIn'
                    },
                    {
                        'name': 'gitHubOAuthAppInfo'
                    },
                    {
                        'name': 'version'
                    },
                    {
                        'name': 'alembicVersion'
                    },
                    {
                        'name': 'isAdmin'
                    },
                    {
                        'name': 'gitHubViewer'
                    },
                    {
                        'name': 'allProducts'
                    },
                    {
                        'name': 'allProductTypes'
                    },
                    {
                        'name': 'allProductRelations'
                    },
                    {
                        'name': 'allProductRelationTypes'
                    },
                    {
                        'name': 'allProductFilePaths'
                    },
                    {
                        'name': 'allFields'
                    },
                    {
                        'name': 'product'
                    },
                    {
                        'name': 'productType'
                    },
                    {
                        'name': 'productRelation'
                    },
                    {
                        'name': 'productRelationType'
                    },
                    {
                        'name': 'field'
                    },
                    {
                        'name': 'node'
                    },
                    {
                        'name': 'allGitHubOrgs'
                    },
                    {
                        'name': 'allGitHubUsers'
                    },
                    {
                        'name': 'allGitHubTokens'
                    },
                    {
                        'name': 'allLogs'
                    },
                    {
                        'name': 'log'
                    }
                ]
            },
            'subscriptionType': None,
            'types': [
                {
                    'fields': [
                        {
                            'name': 'webConfig'
                        },
                        {
                            'name': 'isSignedIn'
                        },
                        {
                            'name': 'gitHubOAuthAppInfo'
                        },
                        {
                            'name': 'version'
                        },
                        {
                            'name': 'alembicVersion'
                        },
                        {
                            'name': 'isAdmin'
                        },
                        {
                            'name': 'gitHubViewer'
                        },
                        {
                            'name': 'allProducts'
                        },
                        {
                            'name': 'allProductTypes'
                        },
                        {
                            'name': 'allProductRelations'
                        },
                        {
                            'name': 'allProductRelationTypes'
                        },
                        {
                            'name': 'allProductFilePaths'
                        },
                        {
                            'name': 'allFields'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'productType'
                        },
                        {
                            'name': 'productRelation'
                        },
                        {
                            'name': 'productRelationType'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'allGitHubOrgs'
                        },
                        {
                            'name': 'allGitHubUsers'
                        },
                        {
                            'name': 'allGitHubTokens'
                        },
                        {
                            'name': 'allLogs'
                        },
                        {
                            'name': 'log'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'QueryAdmin'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'json'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'WebConfig'
                },
                {
                    'fields': [
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'INTERFACE',
                    'name': 'Node'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'ID'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'String'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Boolean'
                },
                {
                    'fields': [
                        {
                            'name': 'clientId'
                        },
                        {
                            'name': 'authorizeUrl'
                        },
                        {
                            'name': 'redirectUri'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOAuthAppInfo'
                },
                {
                    'fields': [
                        {
                            'name': 'userId'
                        },
                        {
                            'name': 'gitHubId'
                        },
                        {
                            'name': 'login'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'avatarUrl'
                        },
                        {
                            'name': 'url'
                        },
                        {
                            'name': 'postedProducts'
                        },
                        {
                            'name': 'updatedProducts'
                        },
                        {
                            'name': 'tokens'
                        },
                        {
                            'name': 'memberships'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubUser'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'hasNextPage'
                        },
                        {
                            'name': 'hasPreviousPage'
                        },
                        {
                            'name': 'startCursor'
                        },
                        {
                            'name': 'endCursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'PageInfo'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'timePosted'
                        },
                        {
                            'name': 'postingGitHubUserId'
                        },
                        {
                            'name': 'timeUpdated'
                        },
                        {
                            'name': 'updatingGitHubUserId'
                        },
                        {
                            'name': 'note'
                        },
                        {
                            'name': 'type_'
                        },
                        {
                            'name': 'postingGitHubUser'
                        },
                        {
                            'name': 'updatingGitHubUser'
                        },
                        {
                            'name': 'paths'
                        },
                        {
                            'name': 'relations'
                        },
                        {
                            'name': 'attributesUnicodeText'
                        },
                        {
                            'name': 'attributesBoolean'
                        },
                        {
                            'name': 'attributesInteger'
                        },
                        {
                            'name': 'attributesFloat'
                        },
                        {
                            'name': 'attributesDate'
                        },
                        {
                            'name': 'attributesDateTime'
                        },
                        {
                            'name': 'attributesTime'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Product'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Int'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'DateTime'
                },
                {
                    'fields': [
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'order'
                        },
                        {
                            'name': 'indefArticle'
                        },
                        {
                            'name': 'singular'
                        },
                        {
                            'name': 'plural'
                        },
                        {
                            'name': 'icon'
                        },
                        {
                            'name': 'products'
                        },
                        {
                            'name': 'fields'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'ProductFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'TypeFieldAssociationConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'TypeFieldAssociationEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'order'
                        },
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'type_'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'attributesUnicodeText'
                        },
                        {
                            'name': 'attributesBoolean'
                        },
                        {
                            'name': 'attributesInteger'
                        },
                        {
                            'name': 'attributesFloat'
                        },
                        {
                            'name': 'attributesDate'
                        },
                        {
                            'name': 'attributesDateTime'
                        },
                        {
                            'name': 'attributesTime'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'TypeFieldAssociation'
                },
                {
                    'fields': [
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'attributesUnicodeText'
                        },
                        {
                            'name': 'attributesBoolean'
                        },
                        {
                            'name': 'attributesInteger'
                        },
                        {
                            'name': 'attributesFloat'
                        },
                        {
                            'name': 'attributesDate'
                        },
                        {
                            'name': 'attributesDateTime'
                        },
                        {
                            'name': 'attributesTime'
                        },
                        {
                            'name': 'entryTypes'
                        },
                        {
                            'name': 'id'
                        },
                        {
                            'name': 'type_'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Field'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeUnicodeTextConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeUnicodeTextEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeUnicodeText'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeUnicodeTextSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeBooleanConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeBooleanEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeBoolean'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeBooleanSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeIntegerConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeIntegerEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeInteger'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeIntegerSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeFloatConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeFloatEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeFloat'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Float'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeFloatSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDate'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeDateSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateTimeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateTimeEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateTime'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeDateTimeSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeTimeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeTimeEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeTime'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeTimeSortEnum'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'TypeFieldAssociationSortEnum'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'FieldType'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductFilePathConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductFilePathEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'pathId'
                        },
                        {
                            'name': 'path'
                        },
                        {
                            'name': 'note'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductFilePath'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductFilePathSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'relationId'
                        },
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'selfProductId'
                        },
                        {
                            'name': 'otherProductId'
                        },
                        {
                            'name': 'reverseRelationId'
                        },
                        {
                            'name': 'type_'
                        },
                        {
                            'name': 'self_'
                        },
                        {
                            'name': 'other'
                        },
                        {
                            'name': 'reverse'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelation'
                },
                {
                    'fields': [
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'reverseTypeId'
                        },
                        {
                            'name': 'indefArticle'
                        },
                        {
                            'name': 'singular'
                        },
                        {
                            'name': 'plural'
                        },
                        {
                            'name': 'reverse'
                        },
                        {
                            'name': 'relations'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationType'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductRelationSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubTokenConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubTokenEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'tokenId'
                        },
                        {
                            'name': 'scope'
                        },
                        {
                            'name': 'userId'
                        },
                        {
                            'name': 'timeCreated'
                        },
                        {
                            'name': 'user'
                        },
                        {
                            'name': 'id'
                        },
                        {
                            'name': 'tokenMasked'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubToken'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'GitHubTokenFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'GitHubTokenSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgMembershipConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgMembershipEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'entryId'
                        },
                        {
                            'name': 'orgId'
                        },
                        {
                            'name': 'memberId'
                        },
                        {
                            'name': 'org'
                        },
                        {
                            'name': 'member'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgMembership'
                },
                {
                    'fields': [
                        {
                            'name': 'orgId'
                        },
                        {
                            'name': 'gitHubId'
                        },
                        {
                            'name': 'login'
                        },
                        {
                            'name': 'avatarUrl'
                        },
                        {
                            'name': 'url'
                        },
                        {
                            'name': 'memberships'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrg'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'GitHubOrgMembershipSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductTypeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductTypeEdge'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'ProductTypeFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductTypeSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationTypeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationTypeEdge'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductRelationTypeSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'FieldConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'FieldEdge'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'FieldSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgEdge'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'GitHubOrgSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubUserConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubUserEdge'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'GitHubUserFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'GitHubUserSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'LogConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'LogEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'level'
                        },
                        {
                            'name': 'message'
                        },
                        {
                            'name': 'time'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Log'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'LogSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'authenticateWithGitHub'
                        },
                        {
                            'name': 'createLog'
                        },
                        {
                            'name': 'createProduct'
                        },
                        {
                            'name': 'deleteProduct'
                        },
                        {
                            'name': 'updateProduct'
                        },
                        {
                            'name': 'convertProductType'
                        },
                        {
                            'name': 'createProductFilePath'
                        },
                        {
                            'name': 'deleteProductFilePath'
                        },
                        {
                            'name': 'updateProductFilePath'
                        },
                        {
                            'name': 'createProductRelation'
                        },
                        {
                            'name': 'deleteProductRelation'
                        },
                        {
                            'name': 'createProductRelationTypes'
                        },
                        {
                            'name': 'deleteProductRelationTypes'
                        },
                        {
                            'name': 'updateProductRelationType'
                        },
                        {
                            'name': 'createProductType'
                        },
                        {
                            'name': 'deleteProductType'
                        },
                        {
                            'name': 'updateProductType'
                        },
                        {
                            'name': 'createField'
                        },
                        {
                            'name': 'deleteField'
                        },
                        {
                            'name': 'updateField'
                        },
                        {
                            'name': 'addGitHubOrg'
                        },
                        {
                            'name': 'deleteGitHubOrg'
                        },
                        {
                            'name': 'addGitHubAdminAppToken'
                        },
                        {
                            'name': 'deleteGitHubAdminAppToken'
                        },
                        {
                            'name': 'updateGitHubOrgMemberLists'
                        },
                        {
                            'name': 'deleteLog'
                        },
                        {
                            'name': 'saveWebConfig'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'MutationAdmin'
                },
                {
                    'fields': [
                        {
                            'name': 'authPayload'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthenticateWithGitHub'
                },
                {
                    'fields': [
                        {
                            'name': 'token'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthPayload'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'log'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateLog'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateLogInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'product'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProduct'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductInput'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'RelationInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributesInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeUnicodeTextInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeBooleanInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeIntegerInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeFloatInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeDateInputFields'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Date'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeDateTimeInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeTimeInputFields'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Time'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProduct'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'product'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProduct'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'product'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ConvertProductType'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productFilePath'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductFilePath'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductFilePathInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductFilePath'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productFilePath'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProductFilePath'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductFilePathInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productRelation'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductRelation'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductRelationInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductRelation'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productRelationType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductRelationTypes'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductRelationTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductRelationTypes'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productRelationType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProductRelationType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductRelationTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductType'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProductType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'field'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateField'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateFieldInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteField'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'field'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateField'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateFieldInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'gitHubOrg'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AddGitHubOrg'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteGitHubOrg'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AddGitHubAdminAppToken'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteGitHubAdminAppToken'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateGitHubOrgMemberLists'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteLog'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'webConfig'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'SaveWebconfig'
                },
                {
                    'fields': [
                        {
                            'name': 'types'
                        },
                        {
                            'name': 'queryType'
                        },
                        {
                            'name': 'mutationType'
                        },
                        {
                            'name': 'subscriptionType'
                        },
                        {
                            'name': 'directives'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Schema'
                },
                {
                    'fields': [
                        {
                            'name': 'kind'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'fields'
                        },
                        {
                            'name': 'interfaces'
                        },
                        {
                            'name': 'possibleTypes'
                        },
                        {
                            'name': 'enumValues'
                        },
                        {
                            'name': 'inputFields'
                        },
                        {
                            'name': 'ofType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Type'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__TypeKind'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'args'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Field'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'defaultValue'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__InputValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__EnumValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'locations'
                        },
                        {
                            'name': 'args'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Directive'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__DirectiveLocation'
                }
            ]
        }
    }
}

snapshots['test_schema[private] 1'] = {
    'data': {
        '__schema': {
            'mutationType': {
                'fields': [
                    {
                        'name': 'authenticateWithGitHub'
                    },
                    {
                        'name': 'createLog'
                    },
                    {
                        'name': 'createProduct'
                    },
                    {
                        'name': 'deleteProduct'
                    },
                    {
                        'name': 'updateProduct'
                    },
                    {
                        'name': 'convertProductType'
                    },
                    {
                        'name': 'createProductFilePath'
                    },
                    {
                        'name': 'deleteProductFilePath'
                    },
                    {
                        'name': 'updateProductFilePath'
                    },
                    {
                        'name': 'createProductRelation'
                    },
                    {
                        'name': 'deleteProductRelation'
                    },
                    {
                        'name': 'createProductRelationTypes'
                    },
                    {
                        'name': 'deleteProductRelationTypes'
                    },
                    {
                        'name': 'updateProductRelationType'
                    },
                    {
                        'name': 'createProductType'
                    },
                    {
                        'name': 'deleteProductType'
                    },
                    {
                        'name': 'updateProductType'
                    },
                    {
                        'name': 'createField'
                    },
                    {
                        'name': 'deleteField'
                    },
                    {
                        'name': 'updateField'
                    }
                ]
            },
            'queryType': {
                'fields': [
                    {
                        'name': 'webConfig'
                    },
                    {
                        'name': 'isSignedIn'
                    },
                    {
                        'name': 'gitHubOAuthAppInfo'
                    },
                    {
                        'name': 'version'
                    },
                    {
                        'name': 'alembicVersion'
                    },
                    {
                        'name': 'isAdmin'
                    },
                    {
                        'name': 'gitHubViewer'
                    },
                    {
                        'name': 'allProducts'
                    },
                    {
                        'name': 'allProductTypes'
                    },
                    {
                        'name': 'allProductRelations'
                    },
                    {
                        'name': 'allProductRelationTypes'
                    },
                    {
                        'name': 'allProductFilePaths'
                    },
                    {
                        'name': 'allFields'
                    },
                    {
                        'name': 'product'
                    },
                    {
                        'name': 'productType'
                    },
                    {
                        'name': 'productRelation'
                    },
                    {
                        'name': 'productRelationType'
                    },
                    {
                        'name': 'field'
                    },
                    {
                        'name': 'node'
                    }
                ]
            },
            'subscriptionType': None,
            'types': [
                {
                    'fields': [
                        {
                            'name': 'webConfig'
                        },
                        {
                            'name': 'isSignedIn'
                        },
                        {
                            'name': 'gitHubOAuthAppInfo'
                        },
                        {
                            'name': 'version'
                        },
                        {
                            'name': 'alembicVersion'
                        },
                        {
                            'name': 'isAdmin'
                        },
                        {
                            'name': 'gitHubViewer'
                        },
                        {
                            'name': 'allProducts'
                        },
                        {
                            'name': 'allProductTypes'
                        },
                        {
                            'name': 'allProductRelations'
                        },
                        {
                            'name': 'allProductRelationTypes'
                        },
                        {
                            'name': 'allProductFilePaths'
                        },
                        {
                            'name': 'allFields'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'productType'
                        },
                        {
                            'name': 'productRelation'
                        },
                        {
                            'name': 'productRelationType'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'node'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'QueryPrivate'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'json'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'WebConfig'
                },
                {
                    'fields': [
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'INTERFACE',
                    'name': 'Node'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'ID'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'String'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Boolean'
                },
                {
                    'fields': [
                        {
                            'name': 'clientId'
                        },
                        {
                            'name': 'authorizeUrl'
                        },
                        {
                            'name': 'redirectUri'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOAuthAppInfo'
                },
                {
                    'fields': [
                        {
                            'name': 'userId'
                        },
                        {
                            'name': 'gitHubId'
                        },
                        {
                            'name': 'login'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'avatarUrl'
                        },
                        {
                            'name': 'url'
                        },
                        {
                            'name': 'postedProducts'
                        },
                        {
                            'name': 'updatedProducts'
                        },
                        {
                            'name': 'tokens'
                        },
                        {
                            'name': 'memberships'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubUser'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'hasNextPage'
                        },
                        {
                            'name': 'hasPreviousPage'
                        },
                        {
                            'name': 'startCursor'
                        },
                        {
                            'name': 'endCursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'PageInfo'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'timePosted'
                        },
                        {
                            'name': 'postingGitHubUserId'
                        },
                        {
                            'name': 'timeUpdated'
                        },
                        {
                            'name': 'updatingGitHubUserId'
                        },
                        {
                            'name': 'note'
                        },
                        {
                            'name': 'type_'
                        },
                        {
                            'name': 'postingGitHubUser'
                        },
                        {
                            'name': 'updatingGitHubUser'
                        },
                        {
                            'name': 'paths'
                        },
                        {
                            'name': 'relations'
                        },
                        {
                            'name': 'attributesUnicodeText'
                        },
                        {
                            'name': 'attributesBoolean'
                        },
                        {
                            'name': 'attributesInteger'
                        },
                        {
                            'name': 'attributesFloat'
                        },
                        {
                            'name': 'attributesDate'
                        },
                        {
                            'name': 'attributesDateTime'
                        },
                        {
                            'name': 'attributesTime'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Product'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Int'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'DateTime'
                },
                {
                    'fields': [
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'order'
                        },
                        {
                            'name': 'indefArticle'
                        },
                        {
                            'name': 'singular'
                        },
                        {
                            'name': 'plural'
                        },
                        {
                            'name': 'icon'
                        },
                        {
                            'name': 'products'
                        },
                        {
                            'name': 'fields'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'ProductFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'TypeFieldAssociationConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'TypeFieldAssociationEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'order'
                        },
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'type_'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'attributesUnicodeText'
                        },
                        {
                            'name': 'attributesBoolean'
                        },
                        {
                            'name': 'attributesInteger'
                        },
                        {
                            'name': 'attributesFloat'
                        },
                        {
                            'name': 'attributesDate'
                        },
                        {
                            'name': 'attributesDateTime'
                        },
                        {
                            'name': 'attributesTime'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'TypeFieldAssociation'
                },
                {
                    'fields': [
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'attributesUnicodeText'
                        },
                        {
                            'name': 'attributesBoolean'
                        },
                        {
                            'name': 'attributesInteger'
                        },
                        {
                            'name': 'attributesFloat'
                        },
                        {
                            'name': 'attributesDate'
                        },
                        {
                            'name': 'attributesDateTime'
                        },
                        {
                            'name': 'attributesTime'
                        },
                        {
                            'name': 'entryTypes'
                        },
                        {
                            'name': 'id'
                        },
                        {
                            'name': 'type_'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Field'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeUnicodeTextConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeUnicodeTextEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeUnicodeText'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeUnicodeTextSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeBooleanConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeBooleanEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeBoolean'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeBooleanSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeIntegerConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeIntegerEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeInteger'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeIntegerSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeFloatConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeFloatEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeFloat'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Float'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeFloatSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDate'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeDateSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateTimeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateTimeEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeDateTime'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeDateTimeSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeTimeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeTimeEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'iid'
                        },
                        {
                            'name': 'value'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'typeFieldAssociationIid'
                        },
                        {
                            'name': 'fieldId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'typeFieldAssociation'
                        },
                        {
                            'name': 'field'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AttributeTime'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'AttributeTimeSortEnum'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'TypeFieldAssociationSortEnum'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'FieldType'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductFilePathConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductFilePathEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'pathId'
                        },
                        {
                            'name': 'path'
                        },
                        {
                            'name': 'note'
                        },
                        {
                            'name': 'productId'
                        },
                        {
                            'name': 'product'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductFilePath'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductFilePathSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'relationId'
                        },
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'selfProductId'
                        },
                        {
                            'name': 'otherProductId'
                        },
                        {
                            'name': 'reverseRelationId'
                        },
                        {
                            'name': 'type_'
                        },
                        {
                            'name': 'self_'
                        },
                        {
                            'name': 'other'
                        },
                        {
                            'name': 'reverse'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelation'
                },
                {
                    'fields': [
                        {
                            'name': 'typeId'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'reverseTypeId'
                        },
                        {
                            'name': 'indefArticle'
                        },
                        {
                            'name': 'singular'
                        },
                        {
                            'name': 'plural'
                        },
                        {
                            'name': 'reverse'
                        },
                        {
                            'name': 'relations'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationType'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductRelationSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubTokenConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubTokenEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'tokenId'
                        },
                        {
                            'name': 'scope'
                        },
                        {
                            'name': 'userId'
                        },
                        {
                            'name': 'timeCreated'
                        },
                        {
                            'name': 'user'
                        },
                        {
                            'name': 'id'
                        },
                        {
                            'name': 'tokenMasked'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubToken'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'GitHubTokenFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'GitHubTokenSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgMembershipConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgMembershipEdge'
                },
                {
                    'fields': [
                        {
                            'name': 'entryId'
                        },
                        {
                            'name': 'orgId'
                        },
                        {
                            'name': 'memberId'
                        },
                        {
                            'name': 'org'
                        },
                        {
                            'name': 'member'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrgMembership'
                },
                {
                    'fields': [
                        {
                            'name': 'orgId'
                        },
                        {
                            'name': 'gitHubId'
                        },
                        {
                            'name': 'login'
                        },
                        {
                            'name': 'avatarUrl'
                        },
                        {
                            'name': 'url'
                        },
                        {
                            'name': 'memberships'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOrg'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'GitHubOrgMembershipSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductTypeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductTypeEdge'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'ProductTypeFilter'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductTypeSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationTypeConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ProductRelationTypeEdge'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'ProductRelationTypeSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'pageInfo'
                        },
                        {
                            'name': 'edges'
                        },
                        {
                            'name': 'totalCount'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'FieldConnection'
                },
                {
                    'fields': [
                        {
                            'name': 'node'
                        },
                        {
                            'name': 'cursor'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'FieldEdge'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': 'FieldSortEnum'
                },
                {
                    'fields': [
                        {
                            'name': 'authenticateWithGitHub'
                        },
                        {
                            'name': 'createLog'
                        },
                        {
                            'name': 'createProduct'
                        },
                        {
                            'name': 'deleteProduct'
                        },
                        {
                            'name': 'updateProduct'
                        },
                        {
                            'name': 'convertProductType'
                        },
                        {
                            'name': 'createProductFilePath'
                        },
                        {
                            'name': 'deleteProductFilePath'
                        },
                        {
                            'name': 'updateProductFilePath'
                        },
                        {
                            'name': 'createProductRelation'
                        },
                        {
                            'name': 'deleteProductRelation'
                        },
                        {
                            'name': 'createProductRelationTypes'
                        },
                        {
                            'name': 'deleteProductRelationTypes'
                        },
                        {
                            'name': 'updateProductRelationType'
                        },
                        {
                            'name': 'createProductType'
                        },
                        {
                            'name': 'deleteProductType'
                        },
                        {
                            'name': 'updateProductType'
                        },
                        {
                            'name': 'createField'
                        },
                        {
                            'name': 'deleteField'
                        },
                        {
                            'name': 'updateField'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'MutationPrivate'
                },
                {
                    'fields': [
                        {
                            'name': 'authPayload'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthenticateWithGitHub'
                },
                {
                    'fields': [
                        {
                            'name': 'token'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthPayload'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'log'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateLog'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'level'
                        },
                        {
                            'name': 'message'
                        },
                        {
                            'name': 'time'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Log'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateLogInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'product'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProduct'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductInput'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'RelationInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributesInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeUnicodeTextInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeBooleanInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeIntegerInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeFloatInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeDateInputFields'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Date'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeDateTimeInputFields'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'AttributeTimeInputFields'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Time'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProduct'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'product'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProduct'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'product'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'ConvertProductType'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productFilePath'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductFilePath'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductFilePathInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductFilePath'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productFilePath'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProductFilePath'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductFilePathInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productRelation'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductRelation'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductRelationInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductRelation'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productRelationType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductRelationTypes'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductRelationTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductRelationTypes'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productRelationType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProductRelationType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductRelationTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateProductType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateProductTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteProductType'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'productType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateProductType'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateProductTypeInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'field'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateField'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateFieldInput'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'DeleteField'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'field'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'UpdateField'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'UpdateFieldInput'
                },
                {
                    'fields': [
                        {
                            'name': 'types'
                        },
                        {
                            'name': 'queryType'
                        },
                        {
                            'name': 'mutationType'
                        },
                        {
                            'name': 'subscriptionType'
                        },
                        {
                            'name': 'directives'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Schema'
                },
                {
                    'fields': [
                        {
                            'name': 'kind'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'fields'
                        },
                        {
                            'name': 'interfaces'
                        },
                        {
                            'name': 'possibleTypes'
                        },
                        {
                            'name': 'enumValues'
                        },
                        {
                            'name': 'inputFields'
                        },
                        {
                            'name': 'ofType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Type'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__TypeKind'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'args'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Field'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'defaultValue'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__InputValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__EnumValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'locations'
                        },
                        {
                            'name': 'args'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Directive'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__DirectiveLocation'
                }
            ]
        }
    }
}

snapshots['test_schema[public-no-token] 1'] = {
    'data': {
        '__schema': {
            'mutationType': {
                'fields': [
                    {
                        'name': 'authenticateWithGitHub'
                    },
                    {
                        'name': 'createLog'
                    }
                ]
            },
            'queryType': {
                'fields': [
                    {
                        'name': 'webConfig'
                    },
                    {
                        'name': 'isSignedIn'
                    },
                    {
                        'name': 'gitHubOAuthAppInfo'
                    }
                ]
            },
            'subscriptionType': None,
            'types': [
                {
                    'fields': [
                        {
                            'name': 'webConfig'
                        },
                        {
                            'name': 'isSignedIn'
                        },
                        {
                            'name': 'gitHubOAuthAppInfo'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'QueryPublic'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'json'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'WebConfig'
                },
                {
                    'fields': [
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'INTERFACE',
                    'name': 'Node'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'ID'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'String'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Boolean'
                },
                {
                    'fields': [
                        {
                            'name': 'clientId'
                        },
                        {
                            'name': 'authorizeUrl'
                        },
                        {
                            'name': 'redirectUri'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOAuthAppInfo'
                },
                {
                    'fields': [
                        {
                            'name': 'authenticateWithGitHub'
                        },
                        {
                            'name': 'createLog'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'MutationPublic'
                },
                {
                    'fields': [
                        {
                            'name': 'authPayload'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthenticateWithGitHub'
                },
                {
                    'fields': [
                        {
                            'name': 'token'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthPayload'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'log'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateLog'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'level'
                        },
                        {
                            'name': 'message'
                        },
                        {
                            'name': 'time'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Log'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'DateTime'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateLogInput'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Int'
                },
                {
                    'fields': [
                        {
                            'name': 'types'
                        },
                        {
                            'name': 'queryType'
                        },
                        {
                            'name': 'mutationType'
                        },
                        {
                            'name': 'subscriptionType'
                        },
                        {
                            'name': 'directives'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Schema'
                },
                {
                    'fields': [
                        {
                            'name': 'kind'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'fields'
                        },
                        {
                            'name': 'interfaces'
                        },
                        {
                            'name': 'possibleTypes'
                        },
                        {
                            'name': 'enumValues'
                        },
                        {
                            'name': 'inputFields'
                        },
                        {
                            'name': 'ofType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Type'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__TypeKind'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'args'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Field'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'defaultValue'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__InputValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__EnumValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'locations'
                        },
                        {
                            'name': 'args'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Directive'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__DirectiveLocation'
                }
            ]
        }
    }
}

snapshots['test_schema[public-wrong-token] 1'] = {
    'data': {
        '__schema': {
            'mutationType': {
                'fields': [
                    {
                        'name': 'authenticateWithGitHub'
                    },
                    {
                        'name': 'createLog'
                    }
                ]
            },
            'queryType': {
                'fields': [
                    {
                        'name': 'webConfig'
                    },
                    {
                        'name': 'isSignedIn'
                    },
                    {
                        'name': 'gitHubOAuthAppInfo'
                    }
                ]
            },
            'subscriptionType': None,
            'types': [
                {
                    'fields': [
                        {
                            'name': 'webConfig'
                        },
                        {
                            'name': 'isSignedIn'
                        },
                        {
                            'name': 'gitHubOAuthAppInfo'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'QueryPublic'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'json'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'WebConfig'
                },
                {
                    'fields': [
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'INTERFACE',
                    'name': 'Node'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'ID'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'String'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Boolean'
                },
                {
                    'fields': [
                        {
                            'name': 'clientId'
                        },
                        {
                            'name': 'authorizeUrl'
                        },
                        {
                            'name': 'redirectUri'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'GitHubOAuthAppInfo'
                },
                {
                    'fields': [
                        {
                            'name': 'authenticateWithGitHub'
                        },
                        {
                            'name': 'createLog'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'MutationPublic'
                },
                {
                    'fields': [
                        {
                            'name': 'authPayload'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthenticateWithGitHub'
                },
                {
                    'fields': [
                        {
                            'name': 'token'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'AuthPayload'
                },
                {
                    'fields': [
                        {
                            'name': 'ok'
                        },
                        {
                            'name': 'log'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'CreateLog'
                },
                {
                    'fields': [
                        {
                            'name': 'id_'
                        },
                        {
                            'name': 'level'
                        },
                        {
                            'name': 'message'
                        },
                        {
                            'name': 'time'
                        },
                        {
                            'name': 'id'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': 'Log'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'DateTime'
                },
                {
                    'fields': None,
                    'kind': 'INPUT_OBJECT',
                    'name': 'CreateLogInput'
                },
                {
                    'fields': None,
                    'kind': 'SCALAR',
                    'name': 'Int'
                },
                {
                    'fields': [
                        {
                            'name': 'types'
                        },
                        {
                            'name': 'queryType'
                        },
                        {
                            'name': 'mutationType'
                        },
                        {
                            'name': 'subscriptionType'
                        },
                        {
                            'name': 'directives'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Schema'
                },
                {
                    'fields': [
                        {
                            'name': 'kind'
                        },
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'fields'
                        },
                        {
                            'name': 'interfaces'
                        },
                        {
                            'name': 'possibleTypes'
                        },
                        {
                            'name': 'enumValues'
                        },
                        {
                            'name': 'inputFields'
                        },
                        {
                            'name': 'ofType'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Type'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__TypeKind'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'args'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Field'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'type'
                        },
                        {
                            'name': 'defaultValue'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__InputValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'isDeprecated'
                        },
                        {
                            'name': 'deprecationReason'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__EnumValue'
                },
                {
                    'fields': [
                        {
                            'name': 'name'
                        },
                        {
                            'name': 'description'
                        },
                        {
                            'name': 'locations'
                        },
                        {
                            'name': 'args'
                        }
                    ],
                    'kind': 'OBJECT',
                    'name': '__Directive'
                },
                {
                    'fields': None,
                    'kind': 'ENUM',
                    'name': '__DirectiveLocation'
                }
            ]
        }
    }
}



"""the instance of SQLAlchemy from Flask-SQLAlchemy

"""
from sqlalchemy import MetaData
from flask_sqlalchemy import SQLAlchemy

##__________________________________________________________________||
# https://flask-sqlalchemy.palletsprojects.com/en/2.x/config/#using-custom-metadata-and-naming-conventions
# https://stackoverflow.com/a/56000475/7309855
convention = {
    "ix": 'ix_%(column_0_label)s',
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    # "ck": "ck_%(table_name)s_%(constraint_name)s",
    "ck": "ck_%(table_name)s_%(column_0_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s"
}
metadata = MetaData(naming_convention=convention)

##__________________________________________________________________||
sa = SQLAlchemy(metadata=metadata)
"""the instance of SQLAlchemy from Flask-SQLAlchemy

While the instance is named "sa" here, it is more commonly named "db"
in example code found online.

- API: https://flask-sqlalchemy.palletsprojects.com/en/2.x/api/#flask_sqlalchemy.SQLAlchemy
- The difference from the plain SQLAlchemy:
  https://flask-sqlalchemy.palletsprojects.com/en/2.x/api/#flask_sqlalchemy.SQLAlchemy

"""
##__________________________________________________________________||



"""Operations on DB
"""

from flask import current_app

from ..db.sa import sa

from ..models import (
    GitHubOrg,
    GitHubUser,
    GitHubToken,
    GitHubOrgMembership,
    AccountAdmin
)

from . import call, query

##__________________________________________________________________||
def get_github_oauth_app_info():
    """Return GitHub OAuth App information

    Returns
    -------
    dict
        e.g.,
            {
                'authorize_url': 'https://github.com/login/oauth/authorize',
                'token_url': 'https://github.com/login/oauth/access_token',
                'client_id': '2a868f8903ce767aa372',
                'client_secret': 'a3b5c532fc28f6ceca8fd284635d915300beb7d3',
                'redirect_uri': 'https://example.org/redirect'
            }
    """
    ret = dict(
        authorize_url=current_app.config['GITHUB_AUTH_AUTHORIZE_URL'],
        token_url=current_app.config['GITHUB_AUTH_TOKEN_URL'],
        client_id=current_app.config['GITHUB_AUTH_CLIENT_ID'],
        client_secret=current_app.config['GITHUB_AUTH_CLIENT_SECRET'],
        redirect_uri=current_app.config['GITHUB_AUTH_REDIRECT_URI']
    )
    return ret

##__________________________________________________________________||
def exchange_code_for_token(code):
    """Exchange an OAuth authentication code for a access token

    Parameters
    ----------
    code : str
        An OAuth authentication code, e.g, '7a31b6726dd2927b0af7'

    Returns
    -------
    dict
        e.g.,
            {
                'access_token': 'cc539c02d5c086e200db6aff212ace633a8c5e77',
                'token_type': 'bearer',
                'scope': ''
            }

    """
    oauth_app_info = get_github_oauth_app_info()
    return call.exchange_code_for_token(
        code,
        token_url=oauth_app_info['token_url'],
        client_id=oauth_app_info['client_id'],
        client_secret=current_app.config['GITHUB_AUTH_CLIENT_SECRET'],
        redirect_uri=current_app.config['GITHUB_AUTH_REDIRECT_URI']
        )

##__________________________________________________________________||
def add_org(login):
    if GitHubOrg.query.filter_by(login=login).one_or_none() is not None:
        raise Exception(f'already exists: {login}')
    if not (tokens := GitHubToken.query.all()):
        raise Exception('No tokens available.')
    r = None
    for token in tokens:
        try:
            r = query.org(login, token.token)
        except:
            continue
        break
    if r is None:
        raise Exception(f'Unable to find an org: {login}')
    model = GitHubOrg(
        login=login,
        git_hub_id=r['id'],
        avatar_url=r['avatarUrl'],
        url=r['url']
    )
    sa.session.add(model)
    sa.session.commit()
    return model

def delete_org(login):
    if (model := GitHubOrg.query.filter_by(login=login).one_or_none()) is None:
        raise Exception(f'does not exist: {login}')
    sa.session.delete(model)
    sa.session.commit()

##__________________________________________________________________||
def update_org_member_lists():
    if not (tokens := GitHubToken.query.filter(GitHubToken.scope.like('%read:org%')).all()):
        raise Exception('No tokens with relevant scopes available.')
    if not (orgs := GitHubOrg.query.all()):
        raise Exception('No orgs found.')
    with sa.session.no_autoflush:
        memberships = GitHubOrgMembership.query.all()
        for membership in memberships:
            sa.session.delete(membership)
        for org in orgs:
            edges = None
            for token in tokens:
                try:
                    edges = query.org_members(org.login, token.token)
                except Exception as e:
                    print(e)
                    continue
                break
            if edges is None:
                raise Exception(f'Unable to find an org: {org.login}')
            for edge in edges:
                node = edge['node']
                if (member := GitHubUser.query.filter_by(git_hub_id=node['id']).one_or_none()) is None:
                    member = GitHubUser(git_hub_id=node['id'])
                member.login = node['login']
                member.name = node['name']
                member.avatar_url = node['avatarUrl']
                member.url = node['url']
                membership = GitHubOrgMembership(org=org, member=member)
                sa.session.add(membership)
    sa.session.commit()

##__________________________________________________________________||
def store_token_for_code(code):
    token_dict = exchange_code_for_token(code)
    viewer = query.viewer(token_dict['access_token'])
    if (user_model := GitHubUser.query.filter_by(git_hub_id=viewer['id']).one_or_none()) is None:
        user_model = GitHubUser(git_hub_id=viewer['id'])
    user_model.login = viewer['login']
    user_model.name = viewer['name']
    user_model.avatar_url = viewer['avatarUrl']
    user_model.url = viewer['url']
    token_model = GitHubToken(
        token=token_dict['access_token'],
        scope=token_dict['scope'],
        user=user_model)
    sa.session.add(token_model)
    sa.session.commit()

##__________________________________________________________________||
def authenticate(code):
    """Authenticate a GitHub user with an OAuth authentication code

    This function authenticates a GitHub user with an OAuth
    authentication code. It also checks the membership of GitHub
    organizations and updates the DB.

    Parameters
    ----------
    code : str
        An OAuth authentication code, e.g, '7a31b6726dd2927b0af7'

    Returns
    -------
    dict
        e.g.,
            {
                'access_token': 'cc539c02d5c086e200db6aff212ace633a8c5e77',
                'token_type': 'bearer',
                'scope': ''
            }

    Raises
    ------
    Exception
        If the authentication is unsuccessful.

    """

    token_dict = exchange_code_for_token(code)
    viewer = query.viewer(token_dict['access_token'])
    account_admin_model = AccountAdmin.query.filter_by(git_hub_login=viewer['login']).one_or_none()
    user_model= GitHubUser.query.filter_by(git_hub_id=viewer['id']).one_or_none()
    if account_admin_model is None:
        if user_model is None:
            raise Exception(f'{viewer["login"]} is not a member.')
        if not user_model.memberships:
            raise Exception(f'{viewer["login"]} is not a member!')
    else:
        if user_model is None:
            user_model = GitHubUser(git_hub_id=viewer['id'])
    user_model.login = viewer['login']
    user_model.name = viewer['name']
    user_model.avatar_url = viewer['avatarUrl']
    user_model.url = viewer['url']
    token_model = GitHubToken(
        token=token_dict['access_token'],
        scope=token_dict['scope'],
        user=user_model)
    sa.session.add(token_model)
    sa.session.commit()
    return token_dict

##__________________________________________________________________||
def get_user_for_token(token):
    """
    """
    user = GitHubUser.query.join(GitHubToken). \
        filter(GitHubToken.token==token). \
        one()
    return user

##__________________________________________________________________||



import pytest

from ....funcs import assert_mutation

from ...gql import MUTATION_CONVERT_PRODUCT_TYPE, QUERY_ALL_PRODUCTS

HEADERS = {
    "Authorization": "Bearer 39d86487d76a84087f1da599c872dac4473e5f07",  # user1
}


##__________________________________________________________________||
params = [
    pytest.param(
        {
            "query": MUTATION_CONVERT_PRODUCT_TYPE,
            "variables": {
                "productId": 1,
                "typeId": 2,
            },
        },
        {"query": QUERY_ALL_PRODUCTS},
        id="convert",
    ),
]


@pytest.mark.parametrize("data_mutation, data_query", params)
@pytest.mark.asyncio
async def test_schema_success(
    app, snapshot, data_mutation, data_query, mock_request_backup_db
):
    success = True
    await assert_mutation(
        app,
        snapshot,
        data_mutation,
        HEADERS,
        data_query,
        HEADERS,
        mock_request_backup_db,
        success,
    )


##__________________________________________________________________||



import json

from acondbs.db.sa import sa
from acondbs.models import WebConfig


SAMPLE_CONFIG_JSON = json.dumps(
    {
        "head_title": "Head Title",
        "toolbar_title": "Toolbar Title",
    },
    indent=2,
)


##__________________________________________________________________||
def test_column(app_empty):
    app = app_empty

    with app.app_context():
        model = WebConfig(json=SAMPLE_CONFIG_JSON)
        sa.session.add(model)
        sa.session.commit()
        id_ = model.id_
        assert id_

    with app.app_context():
        model = WebConfig.query.filter_by(id_=id_).one()
        assert model.json == SAMPLE_CONFIG_JSON


def test_repr(app_empty):
    app = app_empty

    model = WebConfig(json=SAMPLE_CONFIG_JSON)
    repr(model)

    with app.app_context():
        sa.session.add(model)
        sa.session.commit()
        id_ = model.id_

    with app.app_context():
        model = WebConfig.query.filter_by(id_=id_).one()
        repr(model)


##__________________________________________________________________||



import pytest

from acondbs import ops
from acondbs.models import ProductRelationType


##__________________________________________________________________||
def test_fixture(app):
    with app.app_context():
        assert ProductRelationType.query.count() == 2


##__________________________________________________________________||
def test_create(app):
    type_ = {"name": "doctor"}
    reverse = {"name": "patient"}

    with app.app_context():
        count = ProductRelationType.query.count()
        model = ops.create_product_relation_type(type_, reverse)
        ops.commit()
        type_id = model.type_id
        assert type_id

    with app.app_context():
        assert ProductRelationType.query.count() == (count + 2)
        model = ProductRelationType.query.filter_by(type_id=type_id).one()
        assert model.name == "doctor"
        assert model.reverse.name == "patient"


def test_create_self_reverse(app):
    type_ = {"name": "spouse"}

    with app.app_context():
        count = ProductRelationType.query.count()
        model = ops.create_product_relation_type(type_, self_reverse=True)
        ops.commit()
        type_id = model.type_id
        assert type_id

    with app.app_context():
        assert ProductRelationType.query.count() == (count + 1)
        model = ProductRelationType.query.filter_by(type_id=type_id).one()
        assert model.name == "spouse"
        assert model.reverse is model


def test_create_error_both(app):
    type_ = {"name": "doctor"}
    reverse = {"name": "patient"}

    with app.app_context():
        count = ProductRelationType.query.count()
        with pytest.raises(ValueError):
            ops.create_product_relation_type(type_, reverse, self_reverse=True)
            ops.commit()

    with app.app_context():
        assert ProductRelationType.query.count() == count


def test_create_error_neither(app):
    type_ = {"name": "doctor"}

    with app.app_context():
        count = ProductRelationType.query.count()
        with pytest.raises(ValueError):
            ops.create_product_relation_type(type_)
            ops.commit()

    with app.app_context():
        assert ProductRelationType.query.count() == count


##__________________________________________________________________||
def test_update(app):

    type_id = 1

    with app.app_context():
        count = ProductRelationType.query.count()
        model = ops.update_product_relation_type(
            type_id=type_id, name="renamed"
        )
        ops.commit()
        assert model.type_id == type_id

    with app.app_context():
        assert ProductRelationType.query.count() == count
        model = ProductRelationType.query.filter_by(type_id=type_id).one()
        assert model.name == "renamed"


##__________________________________________________________________||
def test_delete(app):

    with app.app_context():
        model = ops.create_product_relation_type(
            type_={"name": "to_be_deleted"},
            reverse={"name": "reverse"},
        )
        ops.commit()
        type_id = model.type_id

    with app.app_context():
        count = ProductRelationType.query.count()
        ops.delete_product_relation_type(type_id=type_id)
        ops.commit()

    with app.app_context():
        assert ProductRelationType.query.count() == count - 2


##__________________________________________________________________||



import graphene

from .. import type_
from .... import ops


##__________________________________________________________________||
class CommonInputFields:
    """Common input fields of mutations for creating and updating file paths"""

    path = graphene.String()
    note = graphene.String()


class CreateProductFilePathInput(graphene.InputObjectType, CommonInputFields):
    product_id = graphene.Int()


class UpdateProductFilePathInput(graphene.InputObjectType, CommonInputFields):
    pass


##__________________________________________________________________||
class CreateProductFilePath(graphene.Mutation):
    class Arguments:
        input = CreateProductFilePathInput(required=True)

    ok = graphene.Boolean()
    productFilePath = graphene.Field(lambda: type_.ProductFilePath)

    def mutate(root, info, input):
        model = ops.create_product_file_path(**input)
        ops.commit()
        ok = True
        return CreateProductFilePath(productFilePath=model, ok=ok)


class UpdateProductFilePath(graphene.Mutation):
    class Arguments:
        path_id = graphene.Int()
        input = UpdateProductFilePathInput(required=True)

    ok = graphene.Boolean()
    productFilePath = graphene.Field(lambda: type_.ProductFilePath)

    def mutate(root, info, path_id, input):
        model = ops.update_product_file_path(path_id, **input)
        ops.commit()
        ok = True
        return UpdateProductFilePath(productFilePath=model, ok=ok)


class DeleteProductFilePath(graphene.Mutation):
    class Arguments:
        path_id = graphene.Int()

    ok = graphene.Boolean()

    def mutate(root, info, path_id):
        ops.delete_product_file_path(path_id)
        ops.commit()
        ok = True
        return DeleteProductFilePath(ok=ok)


##__________________________________________________________________||



"""add a table admin_app_token

Revision ID: d5e539d43045
Revises: ccd0df2c7da1
Create Date: 2020-12-09 13:10:49.501999

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils

# revision identifiers, used by Alembic.
revision = 'd5e539d43045'
down_revision = 'ccd0df2c7da1'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('admin_app_token',
    sa.Column('token_id', sa.Integer(), nullable=False),
    sa.Column('token', sqlalchemy_utils.types.encrypted.encrypted_type.EncryptedType(), nullable=True),
    sa.PrimaryKeyConstraint('token_id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('admin_app_token')
    # ### end Alembic commands ###



import pytest

from ...funcs import assert_mutation

DELETE_GIT_HUB_ADMIN_APP_TOKEN = """
mutation DeleteGitHubAdminAppToken($tokenId: Int!) {
  deleteGitHubAdminAppToken(tokenId: $tokenId) {
    ok
  }
}
"""

ALL_GITHUB_ADMIN_APP_TOKENS = """
{
  allGitHubTokens {
    totalCount
    edges {
      node {
        tokenId
        tokenMasked
        scope
      }
    }
  }
}
"""

##__________________________________________________________________||
params = [
    pytest.param(
        {"query": DELETE_GIT_HUB_ADMIN_APP_TOKEN, "variables": {"tokenId": 1}},
        {"Authorization": "Bearer token1"},
        {"query": ALL_GITHUB_ADMIN_APP_TOKENS},
        {"Authorization": "Bearer token4"},
        id="delete",
    ),
]


@pytest.mark.parametrize(
    "data_mutation, headers_mutation, data_query, headers_query", params
)
@pytest.mark.asyncio
async def test_schema_success(
    app,
    snapshot,
    data_mutation,
    headers_mutation,
    data_query,
    headers_query,
    mock_request_backup_db,
):

    await assert_mutation(
        app,
        snapshot,
        data_mutation,
        headers_mutation,
        data_query,
        headers_query,
        mock_request_backup_db,
        success=True,
    )


##__________________________________________________________________||
params = [
    pytest.param(
        {"query": DELETE_GIT_HUB_ADMIN_APP_TOKEN, "variables": {"tokenId": 999}},
        {"Authorization": "Bearer token1"},
        {"query": ALL_GITHUB_ADMIN_APP_TOKENS},
        {"Authorization": "Bearer token4"},
        id="delete",
    ),
]


@pytest.mark.parametrize(
    "data_mutation, headers_mutation, data_query, headers_query", params
)
@pytest.mark.asyncio
async def test_schema_error(
    app,
    snapshot,
    data_mutation,
    headers_mutation,
    data_query,
    headers_query,
    mock_request_backup_db,
):

    await assert_mutation(
        app,
        snapshot,
        data_mutation,
        headers_mutation,
        data_query,
        headers_query,
        mock_request_backup_db,
        success=False,
    )


##__________________________________________________________________||



from .fragment_product_relation import FRAGMENT_PRODUCT_RELATION

FRAGMENT_PRODUCT_RELATION_CONNECTION = '''
fragment fragmentProductRelationConnection on ProductRelationConnection {
  edges {
    node {
      ...fragmentProductRelation
    }
  }
}
''' + FRAGMENT_PRODUCT_RELATION



"""enter data to field table

Revision ID: 9495d7cb31ec
Revises: 097c6676314c
Create Date: 2021-09-22 19:09:48.564031

"""
from alembic import op

from sqlalchemy.orm.session import Session

import datetime

from sqlalchemy import MetaData
from flask_sqlalchemy import SQLAlchemy

import enum


# revision identifiers, used by Alembic.
revision = '9495d7cb31ec'
down_revision = '097c6676314c'
branch_labels = None
depends_on = None

#
convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    # "ck": "ck_%(table_name)s_%(constraint_name)s",
    "ck": "ck_%(table_name)s_%(column_0_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}
metadata = MetaData(naming_convention=convention)

sa = SQLAlchemy(metadata=metadata)


class ProductType(sa.Model):
    __tablename__ = "product_types"
    type_id = sa.Column(sa.Integer(), primary_key=True)
    name = sa.Column(sa.Text(), nullable=False, unique=True, index=True)
    order = sa.Column(sa.Integer())
    indef_article = sa.Column(sa.Text())
    singular = sa.Column(sa.Text())
    plural = sa.Column(sa.Text())
    icon = sa.Column(sa.Text())


class FieldType(enum.Enum):
    # SQLAlchemy generic types: https://docs.sqlalchemy.org/en/14/core/type_basics.html#generic-types
    UnicodeText = 1
    Boolean = 2
    Integer = 3
    Float = 4
    Date = 5
    DateTime = 6
    Time = 7


class Field(sa.Model):
    __tablename__ = "field"
    field_id = sa.Column(sa.Integer(), primary_key=True)
    name = sa.Column(sa.UnicodeText(), nullable=False)
    type_ = sa.Column(sa.Enum(FieldType), nullable=False)

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.name!r}>"


class TypeFieldAssociation(sa.Model):
    __tablename__ = "type_field_association"
    type_id = sa.Column(
        #
        sa.Integer(),
        sa.ForeignKey("product_types.type_id"),
        primary_key=True,
    )
    field_id = sa.Column(
        #
        sa.Integer(),
        sa.ForeignKey("field.field_id"),
        primary_key=True,
    )
    type_ = sa.relationship(
        "ProductType",
        backref=sa.backref("fields", cascade="all, delete-orphan"),
    )
    field = sa.relationship(
        "Field",
        backref=sa.backref("entry_types", cascade="all, delete-orphan"),
    )

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.type_.name!r} {self.field.name!r}>"


#


def upgrade():
    session = Session(bind=op.get_bind())

    product_types = session.query(ProductType).all()
    if not product_types:
        return

    fields = [
        Field(name="contact", type_=FieldType.UnicodeText),
        Field(name="produced_by", type_=FieldType.UnicodeText),
        Field(name="date_produced", type_=FieldType.Date)
    ]

    with session.no_autoflush:
        for product_type in product_types:
            product_type.fields = [TypeFieldAssociation(field=f) for f in fields]
    session.commit()

def downgrade():
    pass



"""Call GitHub APIs

This module contains functions that actually access to GitHub APIs.
Other modules in the sub-package access to GitHub APIs via functions
in this module.

"""

import requests

API_URL = 'https://api.github.com/graphql'

##__________________________________________________________________||
def call_graphql_api(query, variables=None, token=None):
    """Call a GitHub GraphQL API

    Parameters
    ----------
    query : str
        A GraphQL query, e.g.,
        'query User($login: String!) { user(login: $login) { name } }'
    variables: dict, optional
        Variables for the query, e.g., {"login": "octocat"}
    token: str, optional
        An access token

    Returns
    -------
    dict
        The data field of the query results, e.g.,
        { "user": { "name": "The Octocat" } }

    Raises
    ------
    Exception
        If the query results include the field "errors" or don't include the
        field "data."
    """

    headers = {}
    if token:
      headers['Authorization'] = 'token {}'.format(token)

    json = {'query': query }
    if variables:
        json['variables'] = variables

    # example:
    #   headers = {'Authorization': 'token XXXXXXXXXXXXXXXXXXXX'}
    #   json = {
    #       'query': (
    #           'query User($login: String!) '
    #           '{ user(login: $login) { name } }'
    #       ),
    #       'variables': {"login": "octocat"}
    #   }


    response = requests.post(API_URL, json=json, headers=headers)
    response = response.json()
    # examples:
    #   success:
    #     response = {"data": {"user": {"name": "The Octocat"} } }
    #
    #   error: (note: the 'data' field might not exist.)
    #     response = {
    #         "data": {"user": null},
    #         "errors": [
    #             {
    #                 "type": "NOT_FOUND",
    #                 "path": ["user"],
    #                 "locations": [{"line": 7, "column": 3 }],
    #                 "message": "Could not resolve to a User with the login of 'octocat'."
    #             }
    #         ]
    #     }
    #
    #   bad credentials:
    #     response = {
    #         'message': 'Bad credentials',
    #         'documentation_url': 'https://docs.github.com/graphql'
    #     }

    if 'errors' in response:
      raise Exception(response['errors'])

    if 'data' not in response:
      raise Exception(response)

    return response['data']

##__________________________________________________________________||
def exchange_code_for_token(code, token_url, client_id, client_secret, redirect_uri):
    """exchange a OAuth2 authorization code for an access token

    https://docs.github.com/en/free-pro-team@latest/developers/apps/authorizing-oauth-apps#2-users-are-redirected-back-to-your-site-by-github

    Parameters
    ----------
    code : str
        An OAuth2 authorization code
    token_url: str
        The token URL, i.e., 'https://github.com/login/oauth/access_token'
    client_id: str
        The client ID of the OAuth2 app
    client_secret: str
        The client secret of the OAuth2 app
    redirect_uri: str
        The redirect URI of the OAuth2 app

    Returns
    -------
    dict
        E.g., {'access_token': 'XXXXXXXXXXXXXXXXXXXX', 'token_type': 'bearer', 'scope': 'user'}

    Raises
    ------
    Exception
        If an access token is not obtained, for example, because of a
        bad verification code

    """

    params = {
        'grant_type': 'authorization_code',
        'client_id': client_id,
        'client_secret': client_secret,
        'redirect_uri': redirect_uri,
        'code': code
    }

    headers = {
        'Accept': "application/vnd.github.v3+json, application/json",
    }

    response = requests.post(token_url, json=params, headers=headers)
    response = response.json() # dict
    # examples:
    #   success:
    #     response = {'access_token': 'XXXXXXXXXXXXXXXXXXXX', 'token_type': 'bearer', 'scope': 'user'}
    #
    #   error:
    #     response = {
    #         'error': 'bad_verification_code',
    #         'error_description': 'The code passed is incorrect or expired.',
    #         'error_uri': 'https://docs.github.com/apps/managing-oauth-apps/troubleshooting-oauth-app-access-token-request-errors/#bad-verification-code'
    #     }

    if 'access_token' not in response:
        raise Exception(response)

    return response

##__________________________________________________________________||



from ..fragments import FRAGMENT_PRODUCT_RELATION_TYPE

QUERY_PRODUCT_RELATION_TYPE = """
query ProductRelationType($typeId: Int, $name: String) {
  productRelationType(typeId: $typeId, name: $name) {
    ...fragmentProductRelationType
  }
}
""" + FRAGMENT_PRODUCT_RELATION_TYPE



from setuptools import setup, find_packages
import versioneer

from pathlib import Path

here = Path(__file__).resolve().parent
long_description = here.joinpath("README.md").read_text()

setup(
    name="acondbs",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="A GraphQL server for product DB",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Simons Observatory",
    author_email="so_software@simonsobservatory.org",
    url="https://github.com/simonsobs/acondbs",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    packages=find_packages(exclude=["docs", "tests"]),
    include_package_data=True,
    install_requires=[
        "SQLAlchemy>=1.4",
        "SQLAlchemy-Utils>=0.37",
        "Flask>=2.0",
        "Flask-Cors>=3.0",
        "Flask-GraphQL>=2.0",
        "Flask-Migrate>=3.1",
        "Flask-SQLAlchemy>=2.5",
        "graphene-sqlalchemy>=2.3",
        "graphene-sqlalchemy-filter>=1.10",
        "cryptography>=3.2",
        "gitpython>=3.1",
        "requests>=2.24",
        "a2wsgi>=1.4"
    ],
    extras_require={
        "tests": [
            "pytest>=6.2",
            "pytest-cov>=2.12",
            'pytest-asyncio>=0.14',
            "snapshottest>=0.6",
            "async-asgi-testclient>=1.4.6",
            "sqlparse",
        ]
    },
)



"""delete some attributes

Revision ID: 4398f463d243
Revises: 2910eaefa574
Create Date: 2021-09-20 11:56:40.187352

"""
from alembic import op


from sqlalchemy.orm.session import Session

import datetime

from sqlalchemy import MetaData
from flask_sqlalchemy import SQLAlchemy

# revision identifiers, used by Alembic.
revision = "4398f463d243"
down_revision = "2910eaefa574"
branch_labels = None
depends_on = None


convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    # "ck": "ck_%(table_name)s_%(constraint_name)s",
    "ck": "ck_%(table_name)s_%(column_0_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}
metadata = MetaData(naming_convention=convention)

sa = SQLAlchemy(metadata=metadata)


class GitHubUser(sa.Model):
    __tablename__ = "github_users"
    user_id = sa.Column(sa.Integer(), primary_key=True)
    git_hub_id = sa.Column(sa.Text(), unique=True, nullable=False)
    login = sa.Column(sa.Text(), unique=True, nullable=False)
    name = sa.Column(sa.Text())
    avatar_url = sa.Column(sa.Text())
    url = sa.Column(sa.Text())

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.login!r}>"


class ProductType(sa.Model):
    __tablename__ = "product_types"
    type_id = sa.Column(sa.Integer(), primary_key=True)
    name = sa.Column(sa.Text(), nullable=False, unique=True, index=True)
    order = sa.Column(sa.Integer())
    indef_article = sa.Column(sa.Text())
    singular = sa.Column(sa.Text())
    plural = sa.Column(sa.Text())
    icon = sa.Column(sa.Text())


class Product(sa.Model):
    __tablename__ = "products"
    product_id = sa.Column(sa.Integer(), primary_key=True)
    type_id = sa.Column(sa.ForeignKey("product_types.type_id"), nullable=False)
    type_ = sa.relationship("ProductType", backref=sa.backref("products"))
    name = sa.Column(sa.Text(), nullable=False)
    contact = sa.Column(sa.Text())
    date_produced = sa.Column(sa.Date())
    produced_by = sa.Column(sa.Text())
    time_posted = sa.Column(
        sa.DateTime(), default=lambda: datetime.datetime.now()
    )
    posted_by = sa.Column(sa.Text())
    posting_git_hub_user_id = sa.Column(sa.ForeignKey("github_users.user_id"))
    posting_git_hub_user = sa.relationship(
        "GitHubUser",
        foreign_keys=[posting_git_hub_user_id],
        backref=sa.backref("posted_products", cascade="all"),
    )
    time_updated = sa.Column(sa.DateTime())
    updated_by = sa.Column(sa.Text())
    updating_git_hub_user_id = sa.Column(sa.ForeignKey("github_users.user_id"))
    updating_git_hub_user = sa.relationship(
        "GitHubUser",
        foreign_keys=[updating_git_hub_user_id],
        backref=sa.backref("updated_products", cascade="all"),
    )
    note = sa.Column(sa.Text())
    __table_args__ = (
        sa.UniqueConstraint("type_id", "name", name="_type_id_name"),
    )

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.name!r}>"


class AttributeBase:
    attribute_id = sa.Column(sa.Integer(), primary_key=True)
    name = sa.Column(sa.Text(), nullable=False)

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.name!r}>"


class AttributeText(sa.Model, AttributeBase):
    __tablename__ = "attribute_text"
    product_id = sa.Column(
        sa.Integer(), sa.ForeignKey("products.product_id"), nullable=False
    )
    product = sa.relationship(
        "Product",
        backref=sa.backref("attributes_text", cascade="all, delete-orphan"),
    )
    value = sa.Column(sa.Text())


class AttributeDate(sa.Model, AttributeBase):
    __tablename__ = "attribute_date"
    product_id = sa.Column(
        sa.Integer(), sa.ForeignKey("products.product_id"), nullable=False
    )
    product = sa.relationship(
        "Product",
        backref=sa.backref("attributes_date", cascade="all, delete-orphan"),
    )
    value = sa.Column(sa.Date())


class AttributeDateTime(sa.Model, AttributeBase):
    __tablename__ = "attribute_date_time"
    product_id = sa.Column(
        sa.Integer(), sa.ForeignKey("products.product_id"), nullable=False
    )
    product = sa.relationship(
        "Product",
        backref=sa.backref(
            "attributes_date_time", cascade="all, delete-orphan"
        ),
    )
    value = sa.Column(sa.DateTime())


def upgrade():
    session = Session(bind=op.get_bind())

    columns_text = [
        "name",
        "posted_by",
        "updated_by",
        "note",
    ]
    columns_date = []
    columns_date_time = ["time_posted", "time_updated"]

    products = session.query(Product).all()
    for p in products:
        for c in columns_text:
            for a in session.query(AttributeText).filter_by(name=c):
                session.delete(a)
        for c in columns_date:
            for a in session.query(AttributeDate).filter_by(name=c):
                session.delete(a)
        for c in columns_date_time:
            for a in session.query(AttributeDateTime).filter_by(name=c):
                session.delete(a)
    session.commit()


def downgrade():
    pass



from acondbs import auth

##__________________________________________________________________||
def test_true(app):
    token = '90b2ee5fed25506df04fd37343bb68d1803dd97f' # octcat
    environ_base = {'HTTP_AUTHORIZATION': f'Bearer {token}'}
    with app.test_request_context(environ_base=environ_base):
        assert auth.is_signed_in()
        assert auth.is_admin()

##__________________________________________________________________||
def test_false_not_admin(app):
    token = '0fb8c9e16d6f7c4961c4c49212bf197d79f14080' # dojocat
    environ_base = {'HTTP_AUTHORIZATION': f'Bearer {token}'}
    with app.test_request_context(environ_base=environ_base):
        assert auth.is_signed_in()
        assert not auth.is_admin()

##__________________________________________________________________||
def test_false_wrong_token(app):
    token = '0000000000000000000000000000000000000000'
    environ_base = {'HTTP_AUTHORIZATION': f'Bearer {token}'}
    with app.test_request_context(environ_base=environ_base):
        assert not auth.is_admin()

##__________________________________________________________________||
def test_false_no_token(app):
    environ_base = {}
    with app.test_request_context(environ_base=environ_base):
        assert not auth.is_admin()

##__________________________________________________________________||



from acondbs.db.sa import sa
from acondbs.models import ProductType


##__________________________________________________________________||
def test_column(app_empty):
    app = app_empty

    with app.app_context():
        model = ProductType(
            name="map",
            order=2,
            indef_article="a",
            singular="map",
            plural="maps",
            icon="mdi-map",
        )
        sa.session.add(model)
        sa.session.commit()
        assert model.type_id
        type_id = model.type_id

    with app.app_context():
        model = ProductType.query.filter_by(type_id=type_id).one()
        assert model.type_id == type_id
        assert model.name == "map"
        assert model.order == 2
        assert model.indef_article == "a"
        assert model.singular == "map"
        assert model.plural == "maps"
        assert model.icon == "mdi-map"


def test_repr(app_empty):
    app = app_empty

    model = ProductType(name="map")
    repr(model)

    with app.app_context():
        sa.session.add(model)
        sa.session.commit()
        type_id = model.type_id

    with app.app_context():
        model = ProductType.query.filter_by(type_id=type_id).one()
        repr(model)


##__________________________________________________________________||



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_schema[by-field-id] 1'] = {
    'data': {
        'field': {
            'fieldId': '1',
            'name': 'contact',
            'type_': 'UNICODE_TEXT'
        }
    }
}

snapshots['test_schema[by-name] 1'] = {
    'data': {
        'field': {
            'fieldId': '1',
            'name': 'contact',
            'type_': 'UNICODE_TEXT'
        }
    }
}



import pytest
import unittest.mock as mock

from ...funcs import assert_mutation

ADD_GIT_HUB_ADMIN_APP_TOKEN = """
mutation AddGitHubAdminAppToken($code: String!) {
  addGitHubAdminAppToken(code: $code) {
    ok
  }
}
"""

ALL_GITHUB_ADMIN_APP_TOKENS = """
{
  allGitHubTokens {
    totalCount
    edges {
      node {
        tokenId
        tokenMasked
        scope
      }
    }
  }
}
"""

HEADERS = {"Authorization": "Bearer token1"}  # user1


##__________________________________________________________________||
@pytest.fixture(autouse=True)
def mock_store_token_for_code(monkeypatch):
    y = mock.Mock()
    monkeypatch.setattr("acondbs.schema.github.mutation.store_token_for_code", y)
    yield y


##__________________________________________________________________||
params = [
    pytest.param(
        {"query": ADD_GIT_HUB_ADMIN_APP_TOKEN, "variables": {"code": "code_01234"}},
        {"query": ALL_GITHUB_ADMIN_APP_TOKENS},
        id="one",
    ),
]


@pytest.mark.parametrize("data_mutation, data_query", params)
@pytest.mark.asyncio
async def test_schema_success(
    app,
    snapshot,
    data_mutation,
    data_query,
    mock_request_backup_db,
    mock_store_token_for_code,
):
    await assert_mutation(
        app,
        snapshot,
        data_mutation,
        HEADERS,
        data_query,
        HEADERS,
        mock_request_backup_db,
        success=True,
    )
    snapshot.assert_match(mock_store_token_for_code.call_args_list)


##__________________________________________________________________||
params = [
    pytest.param(
        {"query": ADD_GIT_HUB_ADMIN_APP_TOKEN, "variables": {"code": "code_01234"}},
        {"query": ALL_GITHUB_ADMIN_APP_TOKENS},
        id="one",
    ),
]


@pytest.mark.parametrize("data_mutation, data_query", params)
@pytest.mark.asyncio
async def test_schema_error(
    app,
    snapshot,
    data_mutation,
    data_query,
    mock_request_backup_db,
    mock_store_token_for_code,
):
    mock_store_token_for_code.side_effect = Exception("error")
    await assert_mutation(
        app,
        snapshot,
        data_mutation,
        HEADERS,
        data_query,
        HEADERS,
        mock_request_backup_db,
        success=False,
    )
    snapshot.assert_match(mock_store_token_for_code.call_args_list)


##__________________________________________________________________||



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import GenericRepr, Snapshot


snapshots = Snapshot()

snapshots['test_non_empty 1'] = {
    'account_admins': [
    ],
    'attribute_boolean': [
    ],
    'attribute_date': [
        {
            'field_id': 3,
            'iid': 1,
            'product_id': 1001,
            'type_field_association_iid': 3,
            'value': GenericRepr('datetime.date(2019, 2, 13)')
        },
        {
            'field_id': 3,
            'iid': 2,
            'product_id': 1002,
            'type_field_association_iid': 9,
            'value': GenericRepr('datetime.date(2019, 3, 15)')
        },
        {
            'field_id': 3,
            'iid': 3,
            'product_id': 1010,
            'type_field_association_iid': 6,
            'value': GenericRepr('datetime.date(2018, 1, 1)')
        },
        {
            'field_id': 3,
            'iid': 4,
            'product_id': 1012,
            'type_field_association_iid': 3,
            'value': GenericRepr('datetime.date(2020, 1, 20)')
        },
        {
            'field_id': 3,
            'iid': 5,
            'product_id': 1013,
            'type_field_association_iid': 3,
            'value': GenericRepr('datetime.date(2020, 2, 1)')
        },
        {
            'field_id': 3,
            'iid': 6,
            'product_id': 1070,
            'type_field_association_iid': 6,
            'value': GenericRepr('datetime.date(2019, 3, 4)')
        },
        {
            'field_id': 3,
            'iid': 7,
            'product_id': 1120,
            'type_field_association_iid': 6,
            'value': GenericRepr('datetime.date(2019, 6, 7)')
        },
        {
            'field_id': 3,
            'iid': 8,
            'product_id': 1130,
            'type_field_association_iid': 6,
            'value': GenericRepr('datetime.date(2020, 1, 23)')
        },
        {
            'field_id': 3,
            'iid': 9,
            'product_id': 1150,
            'type_field_association_iid': 6,
            'value': GenericRepr('datetime.date(2020, 2, 7)')
        }
    ],
    'attribute_date_time': [
    ],
    'attribute_float': [
    ],
    'attribute_integer': [
    ],
    'attribute_time': [
    ],
    'attribute_unicode_text': [
        {
            'field_id': 1,
            'iid': 2,
            'product_id': 1001,
            'type_field_association_iid': 1,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 3,
            'product_id': 1001,
            'type_field_association_iid': 2,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 8,
            'product_id': 1002,
            'type_field_association_iid': 7,
            'value': 'abc-def'
        },
        {
            'field_id': 2,
            'iid': 9,
            'product_id': 1002,
            'type_field_association_iid': 8,
            'value': 'abc-def'
        },
        {
            'field_id': 1,
            'iid': 14,
            'product_id': 1010,
            'type_field_association_iid': 4,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 15,
            'product_id': 1010,
            'type_field_association_iid': 5,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 20,
            'product_id': 1012,
            'type_field_association_iid': 1,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 21,
            'product_id': 1012,
            'type_field_association_iid': 2,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 26,
            'product_id': 1013,
            'type_field_association_iid': 1,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 27,
            'product_id': 1013,
            'type_field_association_iid': 2,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 32,
            'product_id': 1070,
            'type_field_association_iid': 4,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 33,
            'product_id': 1070,
            'type_field_association_iid': 5,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 38,
            'product_id': 1120,
            'type_field_association_iid': 4,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 39,
            'product_id': 1120,
            'type_field_association_iid': 5,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 44,
            'product_id': 1130,
            'type_field_association_iid': 4,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 45,
            'product_id': 1130,
            'type_field_association_iid': 5,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 1,
            'iid': 50,
            'product_id': 1150,
            'type_field_association_iid': 4,
            'value': 'pwg-pmn'
        },
        {
            'field_id': 2,
            'iid': 51,
            'product_id': 1150,
            'type_field_association_iid': 5,
            'value': 'pwg-pmn'
        }
    ],
    'field': [
        {
            'field_id': 1,
            'name': 'contact',
            'type_': 'UnicodeText'
        },
        {
            'field_id': 2,
            'name': 'produced_by',
            'type_': 'UnicodeText'
        },
        {
            'field_id': 3,
            'name': 'date_produced',
            'type_': 'Date'
        }
    ],
    'github_org_memberships': [
    ],
    'github_orgs': [
        {
            'avatar_url': 'https://avatars0.githubusercontent.com/u/75631844?v=4',
            'git_hub_id': '012:Organization75631844',
            'login': 'urban-octo-disco',
            'org_id': 1,
            'url': 'https://github.com/urban-octo-disco'
        }
    ],
    'github_tokens': [
        {
            'scope': 'read:org',
            'time_created': GenericRepr('datetime.datetime(2020, 1, 4, 14, 32, 15)'),
            'token': b'aKjGknYDHY39Z2xAaN7+sQ==',
            'token_id': 1,
            'user_id': 1
        }
    ],
    'github_users': [
        {
            'avatar_url': 'https://avatars3.githubusercontent.com/u/583231?v=4',
            'git_hub_id': '04:User583231',
            'login': 'octocat',
            'name': 'The Octocat',
            'url': 'https://github.com/octocat',
            'user_id': 1
        },
        {
            'avatar_url': 'https://avatars0.githubusercontent.com/u/9758946?v=4',
            'git_hub_id': '04:User9758946',
            'login': 'dojocat',
            'name': 'dojocat',
            'url': 'https://github.com/dojocat',
            'user_id': 2
        }
    ],
    'log': [
    ],
    'product_file_paths': [
        {
            'note': '',
            'path': 'nersc:/go/to/my/maps',
            'path_id': 1,
            'product_id': 1001
        },
        {
            'note': 'lat only',
            'path': 'nersc:/go/to/my/maps_v2',
            'path_id': 2,
            'product_id': 1012
        },
        {
            'note': 'lat only',
            'path': 'abcde:/path/to/the/maps_v2',
            'path_id': 3,
            'product_id': 1012
        },
        {
            'note': 'lat only',
            'path': 'nersc:/go/to/my/maps_v3',
            'path_id': 4,
            'product_id': 1013
        },
        {
            'note': '',
            'path': 'BEAM_DEPOT/Beams/20190304',
            'path_id': 5,
            'product_id': 1070
        },
        {
            'note': '',
            'path': 'BEAM_DEPOT/Beams/20190607',
            'path_id': 6,
            'product_id': 1120
        },
        {
            'note': '',
            'path': 'BEAM_DEPOT/Beams/20200123',
            'path_id': 7,
            'product_id': 1130
        },
        {
            'note': '',
            'path': 'BEAM_DEPOT/Beams/20200207',
            'path_id': 8,
            'product_id': 1150
        },
        {
            'note': '',
            'path': 'nersc:/go/to/my/simulations',
            'path_id': 9,
            'product_id': 1002
        },
        {
            'note': '',
            'path': 'abcde:/path/to/the/simulations',
            'path_id': 10,
            'product_id': 1002
        }
    ],
    'product_relation_types': [
        {
            'indef_article': 'a',
            'name': 'parent',
            'plural': 'parents',
            'reverse_type_id': 2,
            'singular': 'parent',
            'type_id': 1
        },
        {
            'indef_article': 'a',
            'name': 'child',
            'plural': 'children',
            'reverse_type_id': 1,
            'singular': 'child',
            'type_id': 2
        },
        {
            'indef_article': 'a',
            'name': 'plaintiff',
            'plural': 'plaintiffs',
            'reverse_type_id': 4,
            'singular': 'plaintiff',
            'type_id': 3
        },
        {
            'indef_article': 'a',
            'name': 'defendant',
            'plural': 'defendants',
            'reverse_type_id': 3,
            'singular': 'defendant',
            'type_id': 4
        }
    ],
    'product_relations': [
        {
            'other_product_id': 1012,
            'relation_id': 1,
            'reverse_relation_id': 2,
            'self_product_id': 1130,
            'type_id': 1
        },
        {
            'other_product_id': 1130,
            'relation_id': 2,
            'reverse_relation_id': 1,
            'self_product_id': 1012,
            'type_id': 2
        },
        {
            'other_product_id': 1013,
            'relation_id': 3,
            'reverse_relation_id': 4,
            'self_product_id': 1150,
            'type_id': 1
        },
        {
            'other_product_id': 1150,
            'relation_id': 4,
            'reverse_relation_id': 3,
            'self_product_id': 1013,
            'type_id': 2
        },
        {
            'other_product_id': 1130,
            'relation_id': 5,
            'reverse_relation_id': 6,
            'self_product_id': 1150,
            'type_id': 1
        },
        {
            'other_product_id': 1150,
            'relation_id': 6,
            'reverse_relation_id': 5,
            'self_product_id': 1130,
            'type_id': 2
        }
    ],
    'product_types': [
        {
            'icon': 'mdi-map',
            'indef_article': 'a',
            'name': 'map',
            'order': 2,
            'plural': 'maps',
            'singular': 'map',
            'type_id': 1
        },
        {
            'icon': 'mdi-spotlight-beam',
            'indef_article': 'a',
            'name': 'beam',
            'order': 3,
            'plural': 'beams',
            'singular': 'beam',
            'type_id': 2
        },
        {
            'icon': 'mdi-creation',
            'indef_article': 'a',
            'name': 'simulation',
            'order': 1,
            'plural': 'simulations',
            'singular': 'simulation',
            'type_id': 3
        },
        {
            'icon': '',
            'indef_article': '',
            'name': 'anchor',
            'order': 4,
            'plural': '',
            'singular': '',
            'type_id': 4
        }
    ],
    'products': [
        {
            'name': 'lat20190213',
            'note': '''- This is a dummy test with a lat map
- This should not depend on any beam''',
            'posting_git_hub_user_id': 1,
            'product_id': 1001,
            'time_posted': GenericRepr('datetime.datetime(2019, 2, 13, 10, 5, 23)'),
            'time_updated': None,
            'type_id': 1,
            'updating_git_hub_user_id': 2
        },
        {
            'name': 'xyz-s1234-20200101',
            'note': '''- note 1
- note 2''',
            'posting_git_hub_user_id': 1,
            'product_id': 1002,
            'time_posted': GenericRepr('datetime.datetime(2019, 3, 15, 9, 11, 25)'),
            'time_updated': None,
            'type_id': 3,
            'updating_git_hub_user_id': None
        },
        {
            'name': '20180101',
            'note': '- test entry',
            'posting_git_hub_user_id': 1,
            'product_id': 1010,
            'time_posted': GenericRepr('datetime.datetime(2018, 1, 1, 15, 32, 10)'),
            'time_updated': None,
            'type_id': 2,
            'updating_git_hub_user_id': None
        },
        {
            'name': 'lat20200120',
            'note': '''- This is a dummy test with a lat map
- A beam depends on this map''',
            'posting_git_hub_user_id': 1,
            'product_id': 1012,
            'time_posted': GenericRepr('datetime.datetime(2020, 1, 20, 18, 10, 5)'),
            'time_updated': None,
            'type_id': 1,
            'updating_git_hub_user_id': 2
        },
        {
            'name': 'lat20200201',
            'note': '''- This is a dummy test with a lat map
- A beam depends on this map''',
            'posting_git_hub_user_id': 1,
            'product_id': 1013,
            'time_posted': GenericRepr('datetime.datetime(2020, 2, 1, 11, 5, 2)'),
            'time_updated': None,
            'type_id': 1,
            'updating_git_hub_user_id': 2
        },
        {
            'name': '20190304',
            'note': '- test entry',
            'posting_git_hub_user_id': 1,
            'product_id': 1070,
            'time_posted': GenericRepr('datetime.datetime(2019, 3, 4, 8, 12, 41)'),
            'time_updated': None,
            'type_id': 2,
            'updating_git_hub_user_id': None
        },
        {
            'name': '20190607',
            'note': '- test entry',
            'posting_git_hub_user_id': 1,
            'product_id': 1120,
            'time_posted': GenericRepr('datetime.datetime(2019, 6, 7, 18, 21, 21)'),
            'time_updated': None,
            'type_id': 2,
            'updating_git_hub_user_id': None
        },
        {
            'name': '20200123',
            'note': '- test entry',
            'posting_git_hub_user_id': 1,
            'product_id': 1130,
            'time_posted': GenericRepr('datetime.datetime(2020, 1, 23, 12, 11, 45)'),
            'time_updated': None,
            'type_id': 2,
            'updating_git_hub_user_id': None
        },
        {
            'name': '20200207',
            'note': '- test entry',
            'posting_git_hub_user_id': 1,
            'product_id': 1150,
            'time_posted': GenericRepr('datetime.datetime(2020, 2, 7, 9, 42, 11)'),
            'time_updated': None,
            'type_id': 2,
            'updating_git_hub_user_id': None
        }
    ],
    'type_field_association': [
        {
            'field_id': 1,
            'iid': 1,
            'order': None,
            'type_id': 1
        },
        {
            'field_id': 2,
            'iid': 2,
            'order': None,
            'type_id': 1
        },
        {
            'field_id': 3,
            'iid': 3,
            'order': None,
            'type_id': 1
        },
        {
            'field_id': 1,
            'iid': 4,
            'order': None,
            'type_id': 2
        },
        {
            'field_id': 2,
            'iid': 5,
            'order': None,
            'type_id': 2
        },
        {
            'field_id': 3,
            'iid': 6,
            'order': None,
            'type_id': 2
        },
        {
            'field_id': 1,
            'iid': 7,
            'order': None,
            'type_id': 3
        },
        {
            'field_id': 2,
            'iid': 8,
            'order': None,
            'type_id': 3
        },
        {
            'field_id': 3,
            'iid': 9,
            'order': None,
            'type_id': 3
        },
        {
            'field_id': 1,
            'iid': 10,
            'order': None,
            'type_id': 4
        },
        {
            'field_id': 2,
            'iid': 11,
            'order': None,
            'type_id': 4
        },
        {
            'field_id': 3,
            'iid': 12,
            'order': None,
            'type_id': 4
        }
    ],
    'web_config': [
        {
            'id_': 1,
            'json': '{"headTitle": "Product DB (test)", "toolbarTitle": "Product DB (test)", "devtoolLoadingstate": true, "productCreationDialog": false, "productUpdateDialog": true, "productDeletionDialog": false}'
        }
    ]
}
